"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TrimEmptyDescriptionsPlugin = void 0;
require("graphile-config");
/**
 * Sets the 'description' attribute to null if it's the empty string; useful
 * for cleaning up the schema as printing a schema with a lot of empty string
 * descriptions is u.g.l.y.
 */
function rmEmptyTypeDescription(type) {
    // TODO: should we trim before comparison?
    if (type.description === "") {
        type.description = null;
    }
    return type;
}
/**
 * Sets the 'description' attribute to null if it's the empty string; useful
 * for cleaning up the schema as printing a schema with a lot of empty string
 * descriptions is u.g.l.y.
 */
function rmEmptyFieldDescription(field) {
    if (field.description === "") {
        field.description = null;
    }
    return field;
}
/**
 * Sets the 'description' attribute to null if it's the empty string; useful
 * for cleaning up the schema as printing a schema with a lot of empty string
 * descriptions is u.g.l.y.
 */
function rmEmptyArgDescriptions(args) {
    if (!args) {
        return args;
    }
    for (const argName in args) {
        const arg = args[argName];
        if (arg.description === "") {
            arg.description = null;
        }
    }
    return args;
}
exports.TrimEmptyDescriptionsPlugin = {
    name: "TrimEmptyDescriptionsPlugin",
    description: "Trims the 'description' property from types, field and args that have an empty description",
    version: "1.0.0",
    schema: {
        hooks: {
            GraphQLObjectType: rmEmptyTypeDescription,
            GraphQLObjectType_fields_field: rmEmptyFieldDescription,
            GraphQLObjectType_fields_field_args: rmEmptyArgDescriptions,
            GraphQLInputObjectType: rmEmptyTypeDescription,
            GraphQLInputObjectType_fields_field: rmEmptyFieldDescription,
            GraphQLUnionType: rmEmptyTypeDescription,
            GraphQLInterfaceType: rmEmptyTypeDescription,
            GraphQLEnumType: rmEmptyTypeDescription,
        },
    },
};
//# sourceMappingURL=TrimEmptyDescriptionsPlugin.js.map